@@ -26,7 +26,7 @@
  * ----------------------------------------------------
  *
  * - Symbols should begin with a prefix indicating in which module they are
- *   used, such as INTF_, VOUT_ or ADEC_.
+ *   used, such as INTF_, VOUT_ or AOUT_.
  */
 
 /*****************************************************************************
@@ -36,9 +36,6 @@
 #define CLOCK_FREQ 1000000
 
 
-/* Automagically spawn audio and video decoder threads */
-#define AUTO_SPAWN
-
 /* When creating or destroying threads in blocking mode, delay to poll thread
  * status */
 #define THREAD_SLEEP                    ((mtime_t)(0.010*CLOCK_FREQ))
@@ -58,6 +55,7 @@
 #  define CONFIG_DIR                    ".vlc"
 #endif
 #define CONFIG_FILE                     "vlcrc"
+#define PLUGINSCACHE_DIR                "cache"
 
 /*****************************************************************************
  * Interface configuration
@@ -88,30 +86,6 @@
  * General limitations
  */
 
-/* Maximum number of input threads - this value is used exclusively by
- * interface, and is in fact an interface limitation */
-#define INPUT_MAX_THREADS               10
-
-/* Maximum size of a data packet (128 kB) */
-#define INPUT_MAX_PACKET_SIZE           131072
-
-/* Maximum length of a pre-parsed chunk (4 MB) */
-#define INPUT_PREPARSE_LENGTH           4194304
-
-/* Maximum length of a hostname or source name */
-#define INPUT_MAX_SOURCE_LENGTH         100
-
-/* Maximum memory the input is allowed to use (20 MB) */
-#define INPUT_MAX_ALLOCATION            20971520
-
-/*
- * Channel method
- */
-
-/* Delay between channel changes - this is required to avoid flooding the
- * channel server */
-#define INPUT_CHANNEL_CHANGE_DELAY      (mtime_t)(5*CLOCK_FREQ)
-
 /* Duration between the time we receive the data packet, and the time we will
  * mark it to be presented */
 #define DEFAULT_PTS_DELAY               (mtime_t)(.3*CLOCK_FREQ)
@@ -131,9 +105,6 @@
  * Audio configuration
  *****************************************************************************/
 
-/* Maximum number of audio output threads */
-#define AOUT_MAX_THREADS                10
-
 /* Volume */
 #define AOUT_VOLUME_DEFAULT             256
 #define AOUT_VOLUME_STEP                32
@@ -169,9 +140,6 @@
  * Video configuration
  *****************************************************************************/
 
-/* Maximum number of video output threads */
-#define VOUT_MAX_THREADS                256
-
 /*
  * Default settings for video output threads
  */
@@ -188,7 +156,11 @@
 
 /* Video heap size - remember that a decompressed picture is big
  * (~1 Mbyte) before using huge values */
-#define VOUT_MAX_PICTURES               8
+#ifdef OPTIMIZE_MEMORY
+#   define VOUT_MAX_PICTURES               5
+#else
+#   define VOUT_MAX_PICTURES               8
+#endif
 
 /* Minimum number of direct pictures the video output will accept without
  * creating additional pictures in system memory */
@@ -197,13 +169,6 @@
 /* Number of simultaneous subpictures */
 #define VOUT_MAX_SUBPICTURES            8
 
-/* Maximum number of active areas in a rendering buffer. Active areas are areas
- * of the picture which need to be cleared before re-using the buffer. If a
- * picture, including its many additions such as subtitles, additionnal user
- * informations and interface, has too many active areas, some of them are
- * joined. */
-#define VOUT_MAX_AREAS                  5
-
 /* Statistics are displayed every n loops (=~ pictures) */
 #define VOUT_STATS_NB_LOOPS             100
 
@@ -224,7 +189,7 @@
 
 /* Pictures which are VOUT_BOGUS_DELAY or more in advance probably have
  * a bogus PTS and won't be displayed */
-#define VOUT_BOGUS_DELAY                ((mtime_t)(DEFAULT_PTS_DELAY * 3))
+#define VOUT_BOGUS_DELAY                ((mtime_t)(DEFAULT_PTS_DELAY * 30))
 
 /* Delay (in microseconds) before an idle screen is displayed */
 #define VOUT_IDLE_DELAY                 (5*CLOCK_FREQ)
@@ -244,38 +209,6 @@
 /* The default video output window title */
 #define VOUT_TITLE                      "VLC"
 
-/*****************************************************************************
- * Video parser configuration
- *****************************************************************************/
-
-#define VPAR_IDLE_SLEEP                 ((mtime_t)(0.010*CLOCK_FREQ))
-
-/* Optimization level, from 0 to 2 - 1 is generally a good compromise. Remember
- * that raising this level dramatically lengthens the compilation time. */
-#if defined( HAVE_RELEASE )
-#   define VPAR_OPTIM_LEVEL             2
-#else
-#   define VPAR_OPTIM_LEVEL             1
-#endif
-
-/* Maximum number of macroblocks in a picture. */
-#define MAX_MB                          2048
-
-/*****************************************************************************
- * Video decoder configuration
- *****************************************************************************/
-
-#define VDEC_IDLE_SLEEP                 ((mtime_t)(0.100*CLOCK_FREQ))
-
-/* Maximum range of values out of the IDCT + motion compensation. */
-#define VDEC_CROPRANGE                  2048
-
-/* No SMP by default, since it slows down things on non-smp machines. */
-#define VDEC_SMP_DEFAULT                0
-
-/* Nice increments for decoders -- necessary for x11 scheduling */
-#define VDEC_NICE                       3
-
 /*****************************************************************************
  * Messages and console interfaces configuration
  *****************************************************************************/
