@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vout_synchro.h: frame-dropping structures
  *****************************************************************************
- * Copyright (C) 1999-2003 VideoLAN
- * $Id: vout_synchro.h,v 1.4 2004/01/25 18:17:08 zorglub Exp $
+ * Copyright (C) 1999-2005 VideoLAN
+ * $Id: vout_synchro.h 11086 2005-05-20 18:00:02Z massiot $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Jean-Marc Dressler <polux@via.ecp.fr>
@@ -36,6 +36,8 @@ struct vout_synchro_t
     vout_thread_t * p_vout;
     int             i_frame_rate;
     int             i_current_rate;
+    vlc_bool_t      b_no_skip;
+    vlc_bool_t      b_quiet;
 
     /* date of the beginning of the decoding of the current picture */
     mtime_t         decoding_start;
@@ -86,10 +88,10 @@ struct vout_synchro_t
 VLC_EXPORT( vout_synchro_t *, __vout_SynchroInit, ( vlc_object_t *, int ) );
 VLC_EXPORT( void, vout_SynchroRelease,        ( vout_synchro_t * ) );
 VLC_EXPORT( void, vout_SynchroReset,          ( vout_synchro_t * ) );
-VLC_EXPORT( vlc_bool_t, vout_SynchroChoose,   ( vout_synchro_t *, int, int ) );
+VLC_EXPORT( vlc_bool_t, vout_SynchroChoose,   ( vout_synchro_t *, int, int, vlc_bool_t ) );
 VLC_EXPORT( void, vout_SynchroTrash,          ( vout_synchro_t * ) );
 VLC_EXPORT( void, vout_SynchroDecode,         ( vout_synchro_t * ) );
 VLC_EXPORT( void, vout_SynchroEnd,            ( vout_synchro_t *, int, vlc_bool_t ) );
 VLC_EXPORT( mtime_t, vout_SynchroDate,        ( vout_synchro_t * ) );
-VLC_EXPORT( void, vout_SynchroNewPicture,     ( vout_synchro_t *, int, int, mtime_t, mtime_t, int ) );
+VLC_EXPORT( void, vout_SynchroNewPicture,     ( vout_synchro_t *, int, int, mtime_t, mtime_t, int, vlc_bool_t ) );
 
