@@ -2,7 +2,7 @@
  * vcd.c : VCD input module for vlc
  *****************************************************************************
  * Copyright (C) 2000-2004 VideoLAN
- * $Id: vcd.c 8606 2004-08-31 18:32:54Z hartman $
+ * $Id: vcd.c 10310 2005-03-11 22:36:40Z anil $
  *
  * Author: Johan Bilien <jobi@via.ecp.fr>
  *
@@ -43,9 +43,12 @@ static void Close( vlc_object_t * );
     "value should be set in milliseconds units." )
 
 vlc_module_begin();
+    set_shortname( _("VCD"));
     set_description( _("VCD input") );
     set_capability( "access2", 60 );
     set_callbacks( Open, Close );
+    set_category( CAT_INPUT );
+    set_subcategory( SUBCAT_INPUT_ACCESS );
 
     add_usage_hint( N_("[vcd:][device][@[title][,[chapter]]]") );
     add_integer( "vcd-caching", DEFAULT_PTS_DELAY / 1000, NULL, CACHING_TEXT,
@@ -124,7 +127,6 @@ static int Open( vlc_object_t *p_this )
     /* Open VCD */
     if( !(vcddev = ioctl_Open( p_this, psz_dup )) )
     {
-        msg_Warn( p_access, "could not open %s", psz_dup );
         free( psz_dup );
         return VLC_EGENERIC;
     }
