@@ -1,8 +1,8 @@
 /*****************************************************************************
  * parse.c: SPU parser
  *****************************************************************************
- * Copyright (C) 2000-2001 VideoLAN
- * $Id: parse.c 8758 2004-09-21 20:33:59Z gbazin $
+ * Copyright (C) 2000-2001, 2005 VideoLAN
+ * $Id: parse.c 10538 2005-04-04 01:40:11Z rocky $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -589,7 +589,7 @@ static void Render( decoder_t *p_dec, subpicture_t *p_spu,
     /* Create a new subpicture region */
     memset( &fmt, 0, sizeof(video_format_t) );
     fmt.i_chroma = VLC_FOURCC('Y','U','V','P');
-    fmt.i_aspect = VOUT_ASPECT_FACTOR;
+    fmt.i_aspect = 0; /* 0 means use aspect ratio of background video */
     fmt.i_width = fmt.i_visible_width = p_spu->i_width;
     fmt.i_height = fmt.i_visible_height = p_spu->i_height -
         p_spu_data->i_y_top_offset - p_spu_data->i_y_bottom_offset;
