@@ -1,8 +1,8 @@
 /*****************************************************************************
  * display.c: Gtk+ tools for main interface
  *****************************************************************************
- * Copyright (C) 1999, 2000 VideoLAN
- * $Id: display.c,v 1.10 2003/05/05 16:09:39 gbazin Exp $
+ * Copyright (C) 1999, 2000, 2003 VideoLAN
+ * $Id: display.c 10101 2005-03-02 16:47:31Z robux4 $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Stphane Borel <stef@via.ecp.fr>
@@ -62,13 +62,14 @@ void E_(GtkDisplayDate)( GtkAdjustment *p_adj )
 
     if( p_intf->p_sys->p_input )
     {
-#define p_area p_intf->p_sys->p_input->stream.p_selected_area
-        char psz_time[ OFFSETTOTIME_MAX_SIZE ];
+        char psz_time[ MSTRTIME_MAX_SIZE ];
+        int64_t i_seconds;
+
+        i_seconds = var_GetTime( p_intf->p_sys->p_input, "time" ) / I64C(1000000 );
+        secstotimestr( psz_time, i_seconds );
 
         gtk_frame_set_label( GTK_FRAME( p_intf->p_sys->p_slider_frame ),
-                        input_OffsetToTime( p_intf->p_sys->p_input, psz_time,
-                                   ( p_area->i_size * p_adj->value ) / 100 ) );
-#undef p_area
+                             psz_time );
      }
 }
 
@@ -215,7 +216,8 @@ gint E_(GtkModeManage)( intf_thread_t * p_intf )
  * GtkHideTooltips: show or hide the tooltips depending on the configuration
  *                  option gnome-tooltips
  *****************************************************************************/
-void E_(GtkHideTooltips)( vlc_object_t *p_this )
+int E_(GtkHideTooltips)( vlc_object_t *p_this, const char *psz_name,
+                         vlc_value_t oldval, vlc_value_t val, void *p_data )
 {
     intf_thread_t *p_intf;
     int i_index;
@@ -244,6 +246,7 @@ void E_(GtkHideTooltips)( vlc_object_t *p_this )
     }
 
     vlc_list_release( p_list );
+    return VLC_SUCCESS;
 }
 
 #ifdef MODULE_NAME_IS_gnome
@@ -253,7 +256,8 @@ void E_(GtkHideTooltips)( vlc_object_t *p_this )
  *****************************************************************************
  * FIXME: GNOME only because of missing icons in gtk interface
  *****************************************************************************/
-void GtkHideToolbarText( vlc_object_t *p_this )
+int GtkHideToolbarText( vlc_object_t *p_this, const char *psz_name,
+                        vlc_value_t oldval, vlc_value_t val, void *p_data )
 {
     GtkToolbarStyle style;
     GtkToolbar * p_toolbar;
@@ -281,5 +285,6 @@ void GtkHideToolbarText( vlc_object_t *p_this )
     }
 
     vlc_list_release( p_list );
+    return VLC_SUCCESS;
 }
 #endif
