@@ -1,8 +1,8 @@
 /*****************************************************************************
- * open.h: MacOS X plugin for vlc
+ * open.h: MacOS X module for vlc
  *****************************************************************************
- * Copyright (C) 2002-2003 VideoLAN
- * $Id: open.h,v 1.16 2003/10/19 23:12:16 hartman Exp $
+ * Copyright (C) 2002-2005 VideoLAN
+ * $Id: open.h 10833 2005-04-26 17:33:24Z bigben $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -69,6 +69,7 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
     IBOutlet id o_net_udpm_port_stp;
     IBOutlet id o_net_http_url;
     IBOutlet id o_net_http_url_lbl;
+    IBOutlet id o_net_timeshift_ckbox;
 
     IBOutlet id o_file_sub_ckbox;
     IBOutlet id o_file_sub_btn_settings;
@@ -82,29 +83,40 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
     IBOutlet id o_file_sub_fps;
     IBOutlet id o_file_sub_fps_lbl;
     IBOutlet id o_file_sub_fps_stp;
+    IBOutlet id o_file_sub_encoding_pop;
+    IBOutlet id o_file_sub_encoding_lbl;
+    IBOutlet id o_file_sub_size_pop;
+    IBOutlet id o_file_sub_size_lbl;
+    IBOutlet id o_file_sub_align_pop;
+    IBOutlet id o_file_sub_align_lbl;
     IBOutlet id o_file_sub_ok_btn;
-    
+    IBOutlet id o_file_sub_font_box;
+    IBOutlet id o_file_sub_file_box;
+
     IBOutlet id o_output_ckbox;
     IBOutlet id o_sout_options;
 }
 
++ (VLCOpen *)sharedInstance;
+
+- (void)setSubPanel;
 - (void)openTarget:(int)i_type;
 - (void)tabView:(NSTabView *)o_tv didSelectTabViewItem:(NSTabViewItem *)o_tvi;
 
-- (IBAction)openFileGeneric:(id)sender;
+- (void)openFileGeneric;
 - (void)openFilePathChanged:(NSNotification *)o_notification;
 - (IBAction)openFileBrowse:(id)sender;
 - (void) pathChosenInPanel: (NSOpenPanel *) sheet withReturn:(int)returnCode contextInfo:(void  *)contextInfo;
 - (IBAction)openFileStreamChanged:(id)sender;
 
-- (IBAction)openDisc:(id)sender;
+- (void)openDisc;
 - (IBAction)openDiscTypeChanged:(id)sender;
 - (IBAction)openDiscStepperChanged:(id)sender;
 - (void)openDiscInfoChanged:(NSNotification *)o_notification;
 - (IBAction)openDiscMenusChanged:(id)sender;
 - (IBAction)openVTSBrowse:(id)sender;
 
-- (IBAction)openNet:(id)sender;
+- (void)openNet;
 - (IBAction)openNetModeChanged:(id)sender;
 - (IBAction)openNetStepperChanged:(id)sender;
 - (void)openNetInfoChanged:(NSNotification *)o_notification;
@@ -120,6 +132,5 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class );
 - (IBAction)panelCancel:(id)sender;
 - (IBAction)panelOk:(id)sender;
 
-- (IBAction)openFile:(id)sender;
-
+- (void)openFile;
 @end
