@@ -1,8 +1,8 @@
 /*****************************************************************************
  * playlist.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2004 VideoLAN
- * $Id: playlist.h 7340 2004-04-13 13:23:26Z bigben $
+ * Copyright (C) 2002-2005 VideoLAN
+ * $Id: playlist.h 11003 2005-05-14 09:20:47Z bigben $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Derk-Jan Hartman <hartman at videolan dot org>
@@ -25,7 +25,7 @@
 /*****************************************************************************
  * VLCPlaylistView interface 
  *****************************************************************************/
-@interface VLCPlaylistView : NSTableView
+@interface VLCPlaylistView : NSOutlineView
 {
 }
 
@@ -36,79 +36,72 @@
  *****************************************************************************/
 @interface VLCPlaylist : NSObject
 {
-    int i_moveRow;
-    bool b_isSortDescending;
+    IBOutlet id o_controller;
 
-    IBOutlet id o_window;
     IBOutlet id o_btn_playlist;
-    IBOutlet id o_table_view;
-
-    IBOutlet id o_status_field;
-    IBOutlet id o_tc_id;
+    IBOutlet id o_playlist_view;
+    IBOutlet id o_outline_view;
     IBOutlet id o_tc_name;
     IBOutlet id o_tc_author;
     IBOutlet id o_tc_duration;
-    IBOutlet id o_tc_sortColumn;
-
-    IBOutlet id o_ctx_menu;
+    IBOutlet id o_status_field;
+    IBOutlet id o_search_field;
+    IBOutlet id o_random_ckb;
+    IBOutlet id o_loop_popup;
     IBOutlet id o_mi_save_playlist;
-    IBOutlet id o_mi_info;
+    IBOutlet id o_ctx_menu;
+
     IBOutlet id o_mi_play;
     IBOutlet id o_mi_delete;
+    IBOutlet id o_mi_info;
     IBOutlet id o_mi_selectall;
-    IBOutlet id o_mi_toggleItemsEnabled;
-    IBOutlet id o_mi_enableGroup;
-    IBOutlet id o_mi_disableGroup;
+    IBOutlet id o_mi_sort_name;
+    IBOutlet id o_mi_sort_author;
 
-    IBOutlet id o_random_ckb;
+    /* "services discovery" menu in the playlist menu */
+    IBOutlet id o_mi_services;
+    IBOutlet id o_mu_services;
 
-    IBOutlet id o_search_keyword;
-    IBOutlet id o_search_button;
-
-    IBOutlet id o_loop_popup;
-
-/*For playlist info window*/
-
-    IBOutlet id o_info_window;
-    IBOutlet id o_uri_lbl;
-    IBOutlet id o_title_lbl;
-    IBOutlet id o_author_lbl;
-    IBOutlet id o_uri_txt;
-    IBOutlet id o_title_txt;
-    IBOutlet id o_author_txt;
-    IBOutlet id o_btn_info_ok;
-    IBOutlet id o_btn_info_cancel;
-    IBOutlet id o_tbv_info;
+    /* "services discovery" menu in the main menu */
+    IBOutlet id o_mm_mi_services;
+    IBOutlet id o_mm_mu_services;
 
     NSImage *o_descendingSortingImage;
     NSImage *o_ascendingSortingImage;
+
+    NSMutableDictionary *o_outline_dict;
+
+    BOOL b_selected_item_met;
+    BOOL b_isSortDescending;
+    int i_current_view;
+    id o_tc_sortColumn;
 }
 
 - (void)initStrings;
+- (void)searchfieldChanged:(NSNotification *)o_notification;
 - (NSMenu *)menuForEvent:(NSEvent *)o_event;
+- (NSOutlineView *)outlineView;
 
-- (IBAction)toggleWindow:(id)sender;
-- (IBAction)savePlaylist:(id)sender;
-- (IBAction)playItem:(id)sender;
-- (IBAction)deleteItems:(id)sender;
-- (IBAction)toggleItemsEnabled:(id)sender;
-- (IBAction)enableGroup:(id)sender;
-- (IBAction)disableGroup:(id)sender;
-- (IBAction)selectAll:(id)sender;
-- (IBAction)searchItem:(id)sender;
 - (IBAction)handlePopUp:(id)sender;
+- (IBAction)searchItem:(id)sender;
+
+- (void)playlistUpdated;
+- (void)playModeUpdated;
+- (void)sortNode:(int)i_mode;
+- (void)updateRowSelection;
+
+- (IBAction)servicesChange:(id)sender;
+- (IBAction)playItem:(id)sender;
+- (IBAction)savePlaylist:(id)sender;
+- (IBAction)deleteItem:(id)sender;
+- (IBAction)selectAll:(id)sender;
+- (IBAction)sortNodeByName:(id)sender;
+- (IBAction)sortNodeByAuthor:(id)sender;
 
 - (void)appendArray:(NSArray*)o_array atPos:(int)i_position enqueue:(BOOL)b_enqueue;
+- (void)appendNodeArray:(NSArray*)o_array inNode:(playlist_item_t *)p_node atPos:(int)i_position inView:(int)i_view enqueue:(BOOL)b_enqueue;
 
-- (void)updateRowSelection;
-- (void)playlistUpdated;
-
-/*For playlist info window*/
-
-- (int)selectedPlaylistItem;
-- (void)deleteGroup:(int)i_id;
-- (NSMutableArray *)selectedPlaylistItemsList;
-- (NSColor *)getColor:(int)i_group;
+- (playlist_item_t *)selectedPlaylistItem;
 
 @end
 
