@@ -2,7 +2,7 @@
  * ctrl_slider.hpp
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: ctrl_slider.hpp 6961 2004-03-05 17:34:23Z sam $
+ * $Id: ctrl_slider.hpp 9934 2005-02-15 13:55:08Z courmisch $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -63,6 +63,9 @@ class CtrlSliderCursor: public CtrlGeneric, public Observer<VarPercent>
         /// Get the text of the tooltip
         virtual UString getTooltipText() const { return m_tooltip; }
 
+        /// Get the type of control (custom RTTI)
+        virtual string getType() const { return "slider_cursor"; }
+
     private:
         /// Finite state machine of the control
         FSM m_fsm;
@@ -125,6 +128,9 @@ class CtrlSliderBg: public CtrlGeneric
         /// Handle an event
         virtual void handleEvent( EvtGeneric &rEvent );
 
+        /// Get the type of control (custom RTTI)
+        virtual string getType() const { return "slider_bg"; }
+
     private:
         /// Cursor of the slider
         CtrlSliderCursor &m_rCursor;
