@@ -2,7 +2,7 @@
  * win32_window.cpp
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: win32_window.cpp 7574 2004-05-01 14:23:40Z asmax $
+ * $Id: win32_window.cpp 10735 2005-04-18 21:21:09Z ipkiss $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -42,7 +42,8 @@ Win32Window::Win32Window( intf_thread_t *pIntf, GenericWindow &rWindow,
                           HINSTANCE hInst, HWND hParentWindow,
                           bool dragDrop, bool playOnDrop,
                           Win32Window *pParentWindow ):
-    OSWindow( pIntf ), m_dragDrop( dragDrop ), m_isLayered( false )
+    OSWindow( pIntf ), m_dragDrop( dragDrop ), m_isLayered( false ),
+    m_pParent( pParentWindow )
 {
     // Create the window
     if( pParentWindow )
@@ -57,7 +58,8 @@ Win32Window::Win32Window( intf_thread_t *pIntf, GenericWindow &rWindow,
     {
         // Normal window
         m_hWnd = CreateWindowEx( WS_EX_TOOLWINDOW, "SkinWindowClass",
-            "default name", WS_POPUP, CW_USEDEFAULT, CW_USEDEFAULT,
+            "default name", WS_POPUP | WS_CLIPCHILDREN,
+            CW_USEDEFAULT, CW_USEDEFAULT,
             CW_USEDEFAULT, CW_USEDEFAULT, hParentWindow, 0, hInst, NULL );
     }
 
@@ -80,16 +82,21 @@ Win32Window::Win32Window( intf_thread_t *pIntf, GenericWindow &rWindow,
         RegisterDragDrop( m_hWnd, m_pDropTarget );
     }
 
-    // XXX Set this window as the vout
-    if( pParentWindow )
+    // Set this window as a vout
+    if( m_pParent )
     {
-        VlcProc::instance( getIntf() )->setVoutWindow( (void*)m_hWnd );
+        VlcProc::instance( getIntf() )->registerVoutWindow( (void*)m_hWnd );
     }
 }
 
 
 Win32Window::~Win32Window()
 {
+    if( m_pParent )
+    {
+        VlcProc::instance( getIntf() )->unregisterVoutWindow( (void*)m_hWnd );
+    }
+
     Win32Factory *pFactory = (Win32Factory*)Win32Factory::instance( getIntf() );
     pFactory->m_windowMap[m_hWnd] = NULL;
 
@@ -127,7 +134,8 @@ void Win32Window::moveResize( int left, int top, int width, int height ) const
 
 void Win32Window::raise() const
 {
-    SetWindowPos( m_hWnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE );
+//     SetWindowPos( m_hWnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE );
+    SetForegroundWindow( m_hWnd );
 }
 
 
