@@ -2,7 +2,7 @@
  * avi.c
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: avi.c,v 1.14 2003/08/10 14:21:16 gbazin Exp $
+ * $Id: avi.c 10101 2005-03-02 16:47:31Z robux4 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -27,78 +27,36 @@
 /* TODO: add OpenDML write support */
 
 #include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <string.h>
-#include <errno.h>
-#include <fcntl.h>
 
 #include <vlc/vlc.h>
 #include <vlc/input.h>
 #include <vlc/sout.h>
 
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
-
 #include "codecs.h"
 
-
-#define AVIF_HASINDEX       0x00000010  // Index at end of file?
-#define AVIF_MUSTUSEINDEX   0x00000020
-#define AVIF_ISINTERLEAVED  0x00000100
-#define AVIF_TRUSTCKTYPE    0x00000800  // Use CKType to find key frames?
-#define AVIF_WASCAPTUREFILE 0x00010000
-#define AVIF_COPYRIGHTED    0x00020000
-
-/* Flags for index */
-#define AVIIF_LIST          0x00000001L /* chunk is a 'LIST' */
-#define AVIIF_KEYFRAME      0x00000010L /* this frame is a key frame.*/
-#define AVIIF_NOTIME        0x00000100L /* this frame doesn't take any time */
-#define AVIIF_COMPUSE       0x0FFF0000L /* these bits are for compressor use */
-
-/*****************************************************************************
- * Exported prototypes
- *****************************************************************************/
-static int     Open   ( vlc_object_t * );
-static void    Close  ( vlc_object_t * );
-
-static int Capability(sout_mux_t *, int , void *, void * );
-static int AddStream( sout_mux_t *, sout_input_t * );
-static int DelStream( sout_mux_t *, sout_input_t * );
-static int Mux      ( sout_mux_t * );
-
-static sout_buffer_t *avi_HeaderCreateRIFF( sout_mux_t * );
-static sout_buffer_t *avi_HeaderCreateidx1( sout_mux_t * );
-
-static void SetFCC( uint8_t *p, char *fcc )
-{
-    p[0] = fcc[0];
-    p[1] = fcc[1];
-    p[2] = fcc[2];
-    p[3] = fcc[3];
-}
-
-static void SetDWLE( uint8_t *p, uint32_t i_dw )
-{
-    p[3] = ( i_dw >> 24 )&0xff;
-    p[2] = ( i_dw >> 16 )&0xff;
-    p[1] = ( i_dw >>  8 )&0xff;
-    p[0] = ( i_dw       )&0xff;
-}
-
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
+static int  Open   ( vlc_object_t * );
+static void Close  ( vlc_object_t * );
+
 vlc_module_begin();
-    set_description( _("Avi muxer") );
+    set_description( _("AVI muxer") );
+    set_category( CAT_SOUT );
+    set_subcategory( SUBCAT_SOUT_MUX );
     set_capability( "sout mux", 5 );
     add_shortcut( "avi" );
     set_callbacks( Open, Close );
 vlc_module_end();
 
-// FIXME FIXME
-#define HDR_SIZE 10240
+
+/*****************************************************************************
+ * Local prototypes
+ *****************************************************************************/
+static int Control( sout_mux_t *, int, va_list );
+static int AddStream( sout_mux_t *, sout_input_t * );
+static int DelStream( sout_mux_t *, sout_input_t * );
+static int Mux      ( sout_mux_t * );
 
 typedef struct avi_stream_s
 {
@@ -106,7 +64,6 @@ typedef struct avi_stream_s
 
     char fcc[4];
 
-    //mtime_t i_first_pts;
     mtime_t i_duration;       // in s
 
     int     i_frames;        // total frame count
@@ -152,6 +109,26 @@ struct sout_mux_sys_t
 
 };
 
+// FIXME FIXME
+#define HDR_SIZE 10240
+
+/* Flags in avih */
+#define AVIF_HASINDEX       0x00000010  // Index at end of file?
+#define AVIF_ISINTERLEAVED  0x00000100
+#define AVIF_TRUSTCKTYPE    0x00000800  // Use CKType to find key frames?
+
+/* Flags for index */
+#define AVIIF_KEYFRAME      0x00000010L /* this frame is a key frame.*/
+
+
+static block_t *avi_HeaderCreateRIFF( sout_mux_t * );
+static block_t *avi_HeaderCreateidx1( sout_mux_t * );
+
+static void SetFCC( uint8_t *p, char *fcc )
+{
+    memcpy( p, fcc, 4 );
+}
+
 /*****************************************************************************
  * Open:
  *****************************************************************************/
@@ -159,7 +136,8 @@ static int Open( vlc_object_t *p_this )
 {
     sout_mux_t      *p_mux = (sout_mux_t*)p_this;
     sout_mux_sys_t  *p_sys = p_mux->p_sys;
-    //sout_buffer_t   *p_hdr;
+
+    msg_Dbg( p_mux, "AVI muxer opened" );
 
     p_sys = malloc( sizeof( sout_mux_sys_t ) );
     p_sys->i_streams = 0;
@@ -168,24 +146,16 @@ static int Open( vlc_object_t *p_this )
 
     p_sys->idx1.i_entry_count = 0;
     p_sys->idx1.i_entry_max = 10000;
-    p_sys->idx1.entry = calloc( p_sys->idx1.i_entry_max, sizeof( avi_idx1_entry_t ) );
+    p_sys->idx1.entry = calloc( p_sys->idx1.i_entry_max,
+                                sizeof( avi_idx1_entry_t ) );
     p_sys->b_write_header = VLC_TRUE;
 
-    msg_Info( p_mux, "Open" );
 
-    p_mux->pf_capacity  = Capability;
+    p_mux->pf_control   = Control;
     p_mux->pf_addstream = AddStream;
     p_mux->pf_delstream = DelStream;
     p_mux->pf_mux       = Mux;
     p_mux->p_sys        = p_sys;
-    p_mux->i_preheader  = 8; // (fourcc,length) header
-
-#if 0
-    /* room to add header at the end */
-    p_hdr = sout_BufferNew( p_mux->p_sout, HDR_SIZE );
-    memset( p_hdr->p_buffer, 0, HDR_SIZE );
-    sout_AccessOutWrite( p_mux->p_access, p_hdr );
-#endif
 
     return VLC_SUCCESS;
 }
@@ -193,20 +163,19 @@ static int Open( vlc_object_t *p_this )
 /*****************************************************************************
  * Close:
  *****************************************************************************/
-
 static void Close( vlc_object_t * p_this )
 {
     sout_mux_t      *p_mux = (sout_mux_t*)p_this;
     sout_mux_sys_t  *p_sys = p_mux->p_sys;
 
-    sout_buffer_t       *p_hdr, *p_idx1;
+    block_t       *p_hdr, *p_idx1;
     int                 i_stream;
 
-    msg_Info( p_mux, "Close" );
+    msg_Dbg( p_mux, "AVI muxer closed" );
 
     /* first create idx1 chunk (write at the end of the stream */
     p_idx1 = avi_HeaderCreateidx1( p_mux );
-    p_sys->i_idx1_size = p_idx1->i_size;
+    p_sys->i_idx1_size = p_idx1->i_buffer;
     sout_AccessOutWrite( p_mux->p_access, p_idx1 );
 
     /* calculate some value for headers creations */
@@ -245,15 +214,30 @@ static void Close( vlc_object_t * p_this )
     sout_AccessOutWrite( p_mux->p_access, p_hdr );
 }
 
-static int Capability( sout_mux_t *p_mux, int i_query, void *p_args, void *p_answer )
+static int Control( sout_mux_t *p_mux, int i_query, va_list args )
 {
+    vlc_bool_t *pb_bool;
+    char **ppsz;
+
    switch( i_query )
    {
-        case SOUT_MUX_CAP_GET_ADD_STREAM_ANY_TIME:
-            *(vlc_bool_t*)p_answer = VLC_FALSE;
-            return( SOUT_MUX_CAP_ERR_OK );
+       case MUX_CAN_ADD_STREAM_WHILE_MUXING:
+           pb_bool = (vlc_bool_t*)va_arg( args, vlc_bool_t * );
+           *pb_bool = VLC_FALSE;
+           return VLC_SUCCESS;
+
+       case MUX_GET_ADD_STREAM_WAIT:
+           pb_bool = (vlc_bool_t*)va_arg( args, vlc_bool_t * );
+           *pb_bool = VLC_TRUE;
+           return VLC_SUCCESS;
+
+       case MUX_GET_MIME:
+           ppsz = (char**)va_arg( args, char ** );
+           *ppsz = strdup( "video/avi" );
+           return VLC_SUCCESS;
+
         default:
-            return( SOUT_MUX_CAP_ERR_UNIMPLEMENTED );
+            return VLC_EGENERIC;
    }
 }
 
@@ -285,22 +269,23 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
 
             p_stream->p_bih = NULL;
 
-            p_stream->p_wf  = malloc( sizeof( WAVEFORMATEX ) + p_input->p_fmt->i_extra_data );
+            p_stream->p_wf  = malloc( sizeof( WAVEFORMATEX ) +
+                                      p_input->p_fmt->i_extra );
 #define p_wf p_stream->p_wf
-            p_wf->cbSize = p_input->p_fmt->i_extra_data;
+            p_wf->cbSize = p_input->p_fmt->i_extra;
             if( p_wf->cbSize > 0 )
             {
                 memcpy( &p_wf[1],
-                        p_input->p_fmt->p_extra_data,
-                        p_input->p_fmt->i_extra_data );
+                        p_input->p_fmt->p_extra,
+                        p_input->p_fmt->i_extra );
             }
-            p_wf->nChannels      = p_input->p_fmt->i_channels;
-            p_wf->nSamplesPerSec = p_input->p_fmt->i_sample_rate;
-            p_wf->nBlockAlign    = p_input->p_fmt->i_block_align;
+            p_wf->nChannels      = p_input->p_fmt->audio.i_channels;
+            p_wf->nSamplesPerSec = p_input->p_fmt->audio.i_rate;
+            p_wf->nBlockAlign    = p_input->p_fmt->audio.i_blockalign;
             p_wf->nAvgBytesPerSec= p_input->p_fmt->i_bitrate / 8;
             p_wf->wBitsPerSample = 0;
 
-            switch( p_input->p_fmt->i_fourcc )
+            switch( p_input->p_fmt->i_codec )
             {
                 case VLC_FOURCC( 'a', '5', '2', ' ' ):
                     p_wf->wFormatTag = WAVE_FORMAT_A52;
@@ -311,11 +296,15 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
                 case VLC_FOURCC( 'w', 'm', 'a', '1' ):
                     p_wf->wFormatTag = WAVE_FORMAT_WMA1;
                     break;
+                case VLC_FOURCC( 'w', 'm', 'a', ' ' ):
                 case VLC_FOURCC( 'w', 'm', 'a', '2' ):
                     p_wf->wFormatTag = WAVE_FORMAT_WMA2;
                     break;
-                case VLC_FOURCC( 'w', 'm', 'a', '3' ):
-                    p_wf->wFormatTag = WAVE_FORMAT_WMA3;
+                case VLC_FOURCC( 'w', 'm', 'a', 'p' ):
+                    p_wf->wFormatTag = WAVE_FORMAT_WMAP;
+                    break;
+                case VLC_FOURCC( 'w', 'm', 'a', 'l' ):
+                    p_wf->wFormatTag = WAVE_FORMAT_WMAL;
                     break;
                     /* raw codec */
                 case VLC_FOURCC( 'u', '8', ' ', ' ' ):
@@ -354,17 +343,19 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
                 p_sys->i_stream_video = p_sys->i_streams;
             }
             p_stream->p_wf  = NULL;
-            p_stream->p_bih = malloc( sizeof( BITMAPINFOHEADER ) + p_input->p_fmt->i_extra_data );
+            p_stream->p_bih = malloc( sizeof( BITMAPINFOHEADER ) +
+                                      p_input->p_fmt->i_extra );
 #define p_bih p_stream->p_bih
-            p_bih->biSize  = sizeof( BITMAPINFOHEADER ) + p_input->p_fmt->i_extra_data;
-            if( p_input->p_fmt->i_extra_data > 0 )
+            p_bih->biSize  = sizeof( BITMAPINFOHEADER ) +
+                             p_input->p_fmt->i_extra;
+            if( p_input->p_fmt->i_extra > 0 )
             {
                 memcpy( &p_bih[1],
-                        p_input->p_fmt->p_extra_data,
-                        p_input->p_fmt->i_extra_data );
+                        p_input->p_fmt->p_extra,
+                        p_input->p_fmt->i_extra );
             }
-            p_bih->biWidth = p_input->p_fmt->i_width;
-            p_bih->biHeight= p_input->p_fmt->i_height;
+            p_bih->biWidth = p_input->p_fmt->video.i_width;
+            p_bih->biHeight= p_input->p_fmt->video.i_height;
             p_bih->biPlanes= 1;
             p_bih->biBitCount       = 24;
             p_bih->biSizeImage      = 0;
@@ -372,13 +363,13 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
             p_bih->biYPelsPerMeter  = 0;
             p_bih->biClrUsed        = 0;
             p_bih->biClrImportant   = 0;
-            switch( p_input->p_fmt->i_fourcc )
+            switch( p_input->p_fmt->i_codec )
             {
                 case VLC_FOURCC( 'm', 'p', '4', 'v' ):
                     p_bih->biCompression = VLC_FOURCC( 'X', 'V', 'I', 'D' );
                     break;
                 default:
-                    p_bih->biCompression = p_input->p_fmt->i_fourcc;
+                    p_bih->biCompression = p_input->p_fmt->i_codec;
                     break;
             }
 #undef p_bih
@@ -389,13 +380,11 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
     p_stream->i_totalsize = 0;
     p_stream->i_frames    = 0;
     p_stream->i_duration  = 0;
-    //p_stream->i_first_pts = 0;
 
     /* fixed later */
     p_stream->f_fps = 25;
     p_stream->i_bitrate = 128 * 1024;
 
-
     p_sys->i_streams++;
     return( VLC_SUCCESS );
 }
@@ -405,8 +394,7 @@ static int DelStream( sout_mux_t *p_mux, sout_input_t *p_input )
 
     msg_Dbg( p_mux, "removing input" );
 
-    free( p_input->p_sys ); p_input->p_sys = NULL;
-
+    free( p_input->p_sys );
     return( 0 );
 }
 
@@ -419,7 +407,7 @@ static int Mux      ( sout_mux_t *p_mux )
 
     if( p_sys->b_write_header )
     {
-        sout_buffer_t *p_hdr;
+        block_t *p_hdr;
 
         msg_Dbg( p_mux, "writing header" );
 
@@ -432,37 +420,42 @@ static int Mux      ( sout_mux_t *p_mux )
     for( i = 0; i < p_mux->i_nb_inputs; i++ )
     {
         int i_count;
-        sout_fifo_t *p_fifo;
+        block_fifo_t *p_fifo;
 
         i_stream = *((int*)p_mux->pp_inputs[i]->p_sys );
         p_stream = &p_sys->stream[i_stream];
 
         p_fifo = p_mux->pp_inputs[i]->p_fifo;
         i_count = p_fifo->i_depth;
-        while( i_count > 0 )
+        while( i_count > 1 )
         {
             avi_idx1_entry_t *p_idx;
-            sout_buffer_t *p_data;
+            block_t *p_data;
 
-            p_data = sout_FifoGet( p_fifo );
+            p_data = block_FifoGet( p_fifo );
+            if( p_fifo->i_depth > 0 )
+            {
+                block_t *p_next = block_FifoShow( p_fifo );
+                p_data->i_length = p_next->i_dts - p_data->i_dts;
+            }
 
             p_stream->i_frames++;
             if( p_data->i_length < 0 )
             {
                 msg_Warn( p_mux, "argg length < 0 l" );
-                sout_BufferDelete( p_mux->p_sout, p_data );
+                block_Release( p_data );
                 i_count--;
                 continue;
             }
             p_stream->i_duration  += p_data->i_length;
-            p_stream->i_totalsize += p_data->i_size;
+            p_stream->i_totalsize += p_data->i_buffer;
 
             /* add idx1 entry for this frame */
             p_idx = &p_sys->idx1.entry[p_sys->idx1.i_entry_count];
             memcpy( p_idx->fcc, p_stream->fcc, 4 );
             p_idx->i_flags = AVIIF_KEYFRAME;
             p_idx->i_pos   = p_sys->i_movi_size + 4;
-            p_idx->i_length= p_data->i_size;
+            p_idx->i_length= p_data->i_buffer;
             p_sys->idx1.i_entry_count++;
             if( p_sys->idx1.i_entry_count >= p_sys->idx1.i_entry_max )
             {
@@ -471,34 +464,20 @@ static int Mux      ( sout_mux_t *p_mux )
                                              p_sys->idx1.i_entry_max * sizeof( avi_idx1_entry_t ) );
             }
 
-
-            if( sout_BufferReallocFromPreHeader( p_mux->p_sout, p_data, 8 ) )
-            {
-                /* there isn't enough data in preheader */
-                sout_buffer_t *p_hdr;
-
-                p_hdr = sout_BufferNew( p_mux->p_sout, 8 );
-                SetFCC( p_hdr->p_buffer, p_stream->fcc );
-                SetDWLE( p_hdr->p_buffer + 4, p_data->i_size );
-
-                sout_AccessOutWrite( p_mux->p_access, p_hdr );
-                p_sys->i_movi_size += p_hdr->i_size;
-
-            }
-            else
+            p_data = block_Realloc( p_data, 8, p_data->i_buffer );
+            if( p_data )
             {
                 SetFCC( p_data->p_buffer, p_stream->fcc );
-                SetDWLE( p_data->p_buffer + 4, p_data->i_size - 8 );
-            }
+                SetDWLE( p_data->p_buffer + 4, p_data->i_buffer - 8 );
 
-            if( p_data->i_size & 0x01 )
-            {
-                sout_BufferRealloc( p_mux->p_sout, p_data, p_data->i_size + 1 );
-                p_data->i_size += 1;
-            }
+                if( p_data->i_buffer & 0x01 )
+                {
+                    p_data = block_Realloc( p_data, 0, p_data->i_buffer + 1 );
+                }
 
-            p_sys->i_movi_size += p_data->i_size;
-            sout_AccessOutWrite( p_mux->p_access, p_data );
+                p_sys->i_movi_size += p_data->i_buffer;
+                sout_AccessOutWrite( p_mux->p_access, p_data );
+            }
 
             i_count--;
         }
@@ -805,16 +784,16 @@ static int avi_HeaderAdd_strl( sout_mux_t *p_mux,
     AVI_BOX_EXIT( 0 );
 }
 
-static sout_buffer_t *avi_HeaderCreateRIFF( sout_mux_t *p_mux )
+static block_t *avi_HeaderCreateRIFF( sout_mux_t *p_mux )
 {
     sout_mux_sys_t      *p_sys = p_mux->p_sys;
-    sout_buffer_t       *p_hdr;
+    block_t       *p_hdr;
     int                 i_stream;
     int                 i_maxbytespersec;
     int                 i_junk;
     buffer_out_t        bo;
 
-    p_hdr = sout_BufferNew( p_mux->p_sout, HDR_SIZE );
+    p_hdr = block_New( p_mux, HDR_SIZE );
     memset( p_hdr->p_buffer, 0, HDR_SIZE );
 
     bo_Init( &bo, HDR_SIZE, p_hdr->p_buffer );
@@ -845,17 +824,17 @@ static sout_buffer_t *avi_HeaderCreateRIFF( sout_mux_t *p_mux )
     return( p_hdr );
 }
 
-static sout_buffer_t * avi_HeaderCreateidx1( sout_mux_t *p_mux )
+static block_t * avi_HeaderCreateidx1( sout_mux_t *p_mux )
 {
     sout_mux_sys_t      *p_sys = p_mux->p_sys;
-    sout_buffer_t       *p_idx1;
+    block_t       *p_idx1;
     uint32_t            i_idx1_size;
     unsigned int        i;
     buffer_out_t        bo;
 
     i_idx1_size = 16 * p_sys->idx1.i_entry_count;
 
-    p_idx1 = sout_BufferNew( p_mux->p_sout, i_idx1_size + 8 );
+    p_idx1 = block_New( p_mux, i_idx1_size + 8 );
     memset( p_idx1->p_buffer, 0, i_idx1_size );
 
     bo_Init( &bo, i_idx1_size, p_idx1->p_buffer );
@@ -873,4 +852,3 @@ static sout_buffer_t * avi_HeaderCreateidx1( sout_mux_t *p_mux )
     return( p_idx1 );
 }
 
-
