@@ -2,7 +2,7 @@
  * pes.c: PES packetizer used by the MPEG multiplexers
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: pes.c,v 1.14 2003/12/10 23:16:04 gbazin Exp $
+ * $Id: pes.c 10731 2005-04-18 15:32:31Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -44,11 +44,11 @@
 #include "pes.h"
 #include "bits.h"
 
-#define PES_PAYLOAD_SIZE_MAX 65500
-
 static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
-                             int i_es_size, int i_stream_id, int i_private_id,
-                             vlc_bool_t b_mpeg2 )
+                             int i_es_size, es_format_t *p_fmt,
+                             int i_stream_id, int i_private_id,
+                             vlc_bool_t b_mpeg2, vlc_bool_t b_data_alignment,
+                             int i_header_size )
 {
     bits_buffer_t bits;
     int     i_extra = 0;
@@ -66,7 +66,7 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
     }
 
 
-    bits_initwrite( &bits, 30, p_hdr );
+    bits_initwrite( &bits, 50, p_hdr );
 
     /* add start code */
     bits_write( &bits, 24, 0x01 );
@@ -92,26 +92,30 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
             {
                 int i_pts_dts;
 
-                if( i_pts > 0 && i_dts > 0 )
+                if( i_pts > 0 && i_dts > 0 &&
+                    ( i_pts != i_dts || ( p_fmt->i_cat == VIDEO_ES &&
+                      p_fmt->i_codec != VLC_FOURCC('m','p','g','v') ) ) )
                 {
-                    bits_write( &bits, 16, i_es_size + i_extra+ 13 );
                     i_pts_dts = 0x03;
+                    if ( !i_header_size ) i_header_size = 0xa;
                 }
                 else if( i_pts > 0 )
                 {
-                    bits_write( &bits, 16, i_es_size  + i_extra + 8 );
                     i_pts_dts = 0x02;
+                    if ( !i_header_size ) i_header_size = 0x5;
                 }
                 else
                 {
-                    bits_write( &bits, 16, i_es_size  + i_extra + 3 );
                     i_pts_dts = 0x00;
+                    if ( !i_header_size ) i_header_size = 0x0;
                 }
 
+                bits_write( &bits, 16, i_es_size + i_extra + 3
+                             + i_header_size ); // size
                 bits_write( &bits, 2, 0x02 ); // mpeg2 id
                 bits_write( &bits, 2, 0x00 ); // pes scrambling control
                 bits_write( &bits, 1, 0x00 ); // pes priority
-                bits_write( &bits, 1, 0x00 ); // data alignement indicator
+                bits_write( &bits, 1, b_data_alignment ); // data alignement indicator
                 bits_write( &bits, 1, 0x00 ); // copyright
                 bits_write( &bits, 1, 0x00 ); // original or copy
 
@@ -122,18 +126,7 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
                 bits_write( &bits, 1, 0x00 ); // additional copy info flag
                 bits_write( &bits, 1, 0x00 ); // pes crc flag
                 bits_write( &bits, 1, 0x00 ); // pes extention flags
-                if( i_pts_dts == 0x03 )
-                {
-                    bits_write( &bits, 8, 0x0a ); // header size -> pts and dts
-                }
-                else if( i_pts_dts == 0x02 )
-                {
-                    bits_write( &bits, 8, 0x05 ); // header size -> pts
-                }
-                else
-                {
-                    bits_write( &bits, 8, 0x00 ); // header size -> 0
-                }
+                bits_write( &bits, 8, i_header_size ); // header size -> pts and dts
 
                 /* write pts */
                 if( i_pts_dts & 0x02 )
@@ -145,6 +138,7 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
                     bits_write( &bits, 1, 0x01 ); // marker
                     bits_write( &bits, 15, i_pts );
                     bits_write( &bits, 1, 0x01 ); // marker
+                    i_header_size -= 0x5;
                 }
                 /* write i_dts */
                 if( i_pts_dts & 0x01 )
@@ -156,13 +150,20 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
                     bits_write( &bits, 1, 0x01 ); // marker
                     bits_write( &bits, 15, i_dts );
                     bits_write( &bits, 1, 0x01 ); // marker
+                    i_header_size -= 0x5;
+                }
+                while ( i_header_size )
+                {
+                    bits_write( &bits, 8, 0xff );
+                    i_header_size--;
                 }
             }
             else /* MPEG1 */
             {
                 int i_pts_dts;
 
-                if( i_pts > 0 && i_dts > 0 )
+                if( i_pts > 0 && i_dts > 0 &&
+                    ( i_pts != i_dts || p_fmt->i_cat == VIDEO_ES ) )
                 {
                     bits_write( &bits, 16, i_es_size + i_extra + 10 /* + stuffing */ );
                     i_pts_dts = 0x03;
@@ -228,13 +229,12 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
     }
 }
 
-int E_( EStoPES )( sout_instance_t *p_sout,
-                   sout_buffer_t **pp_pes,
-                   sout_buffer_t *p_es,
-                   int i_stream_id,
-                   int b_mpeg2 )
+int E_( EStoPES )( sout_instance_t *p_sout, block_t **pp_pes, block_t *p_es,
+                   es_format_t *p_fmt, int i_stream_id,
+                   int b_mpeg2, int b_data_alignment, int i_header_size,
+                   int i_max_pes_size )
 {
-    sout_buffer_t *p_es_sav, *p_pes;
+    block_t *p_pes;
     mtime_t i_pts, i_dts, i_length;
 
     uint8_t *p_data;
@@ -242,7 +242,7 @@ int E_( EStoPES )( sout_instance_t *p_sout,
 
     int     i_private_id = -1;
 
-    uint8_t header[30];     // PES header + extra < 30 (more like 17)
+    uint8_t header[50];     // PES header + extra < 50 (more like 17)
     int     i_pes_payload;
     int     i_pes_header;
 
@@ -258,28 +258,24 @@ int E_( EStoPES )( sout_instance_t *p_sout,
     i_pts = p_es->i_pts <= 0 ? 0 : p_es->i_pts * 9 / 100; // 90000 units clock
     i_dts = p_es->i_dts <= 0 ? 0 : p_es->i_dts * 9 / 100; // 90000 units clock
 
-    i_size = p_es->i_size;
+    i_size = p_es->i_buffer;
     p_data = p_es->p_buffer;
 
     *pp_pes = p_pes = NULL;
-    p_es_sav = p_es;
 
     do
     {
-        i_pes_payload = __MIN( i_size, PES_PAYLOAD_SIZE_MAX );
+        i_pes_payload = __MIN( i_size, (i_max_pes_size ?
+                               i_max_pes_size : PES_PAYLOAD_SIZE_MAX) );
         i_pes_header  = PESHeader( header, i_pts, i_dts, i_pes_payload,
-                                   i_stream_id, i_private_id, b_mpeg2 );
+                                   p_fmt, i_stream_id, i_private_id, b_mpeg2,
+                                   b_data_alignment, i_header_size );
         i_dts = 0; // only first PES has a dts/pts
         i_pts = 0;
 
-        if( p_es  )
+        if( p_es )
         {
-            if( sout_BufferReallocFromPreHeader( p_sout, p_es, i_pes_header ) )
-            {
-                msg_Err( p_sout,
-                         "cannot realloc preheader (should never happen)" );
-                return( -1 );
-            }
+            p_es = block_Realloc( p_es, i_pes_header, p_es->i_buffer );
             /* reuse p_es for first frame */
             *pp_pes = p_pes = p_es;
             /* don't touch i_dts, i_pts, i_length as are already set :) */
@@ -287,8 +283,7 @@ int E_( EStoPES )( sout_instance_t *p_sout,
         }
         else
         {
-            p_pes->p_next = sout_BufferNew( p_sout,
-                                            i_pes_header + i_pes_payload );
+            p_pes->p_next = block_New( p_sout, i_pes_header + i_pes_payload );
             p_pes = p_pes->p_next;
 
             p_pes->i_dts    = 0;
@@ -307,16 +302,10 @@ int E_( EStoPES )( sout_instance_t *p_sout,
 
         i_size -= i_pes_payload;
         p_data += i_pes_payload;
-        p_pes->i_size =  i_pes_header + i_pes_payload;
+        p_pes->i_buffer =  i_pes_header + i_pes_payload;
 
     } while( i_size > 0 );
 
-    /* sav some space */
-    if( p_es_sav->i_size + 10*1024 < p_es_sav->i_buffer_size )
-    {
-        sout_BufferRealloc( p_sout, p_es_sav, p_es_sav->i_size );
-    }
-
     /* Now redate all pes */
     i_dts    = (*pp_pes)->i_dts;
     i_length = (*pp_pes)->i_length / i_pes_count;
@@ -327,7 +316,6 @@ int E_( EStoPES )( sout_instance_t *p_sout,
 
         i_dts += i_length;
     }
-    return( 0 );
+
+    return 0;
 }
-
-
