@@ -2,11 +2,11 @@
  * mpeg4video.c: mpeg 4 video packetizer
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: mpeg4video.c,v 1.22 2003/12/22 02:24:53 sam Exp $
+ * $Id: mpeg4video.c 11349 2005-06-08 07:34:58Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
- *          Gildas Bazin <gbazin@netcourrier.com>
+ *          Gildas Bazin <gbazin@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <vlc/vlc.h>
 #include <vlc/decoder.h>
 #include <vlc/sout.h>
+#include <vlc/input.h>                  /* hmmm, just for INPUT_RATE_DEFAULT */
 
 #include "vlc_bits.h"
 
@@ -41,6 +42,8 @@ static int  Open ( vlc_object_t * );
 static void Close( vlc_object_t * );
 
 vlc_module_begin();
+    set_category( CAT_SOUT );
+    set_subcategory( SUBCAT_SOUT_PACKETIZER );
     set_description( _("MPEG4 video packetizer") );
     set_capability( "packetizer", 50 );
     set_callbacks( Open, Close );
@@ -57,18 +60,31 @@ struct decoder_sys_t
     /*
      * Common properties
      */
-    mtime_t i_pts;
-    mtime_t i_dts;
-
+    mtime_t i_interpolated_pts;
+    mtime_t i_interpolated_dts;
+    mtime_t i_last_ref_pts;
+    mtime_t i_last_time_ref;
+    mtime_t i_time_ref;
+    mtime_t i_last_time;
+    mtime_t i_last_timeincr;
 
     vlc_bool_t  b_vop;
     int         i_buffer;
     int         i_buffer_size;
     uint8_t     *p_buffer;
+    unsigned int i_flags;
+
+    int         i_fps_num;
+    int         i_fps_den;
+    int         i_last_incr;
+    int         i_last_incr_diff;
+
+    vlc_bool_t  b_frame;
 };
 
-static int m4v_FindStartCode( uint8_t **pp_start, uint8_t *p_end );
-static int m4v_VOLParse( es_format_t *fmt, uint8_t *p_vol, int i_vol );
+static int m4v_FindStartCode( uint8_t **, uint8_t * );
+static int m4v_VOLParse( decoder_t *, es_format_t *, uint8_t *, int );
+static int vlc_log2( unsigned int );
 
 #define VIDEO_OBJECT_MASK                       0x01f
 #define VIDEO_OBJECT_LAYER_MASK                 0x00f
@@ -105,6 +121,7 @@ static int Open( vlc_object_t *p_this )
         case VLC_FOURCC( 'm', 'p', '4', 's'):
         case VLC_FOURCC( 'M', 'P', '4', 'S'):
         case VLC_FOURCC( 'm', 'p', '4', 'v'):
+        case VLC_FOURCC( 'M', 'P', '4', 'V'):
         case VLC_FOURCC( 'D', 'I', 'V', 'X'):
         case VLC_FOURCC( 'd', 'i', 'v', 'x'):
         case VLC_FOURCC( 'X', 'V', 'I', 'D'):
@@ -114,6 +131,8 @@ static int Open( vlc_object_t *p_this )
         case VLC_FOURCC( 'd', 'x', '5', '0'):
         case VLC_FOURCC( 0x04, 0,   0,   0):
         case VLC_FOURCC( '3', 'I', 'V', '2'):
+        case VLC_FOURCC( 'm', '4', 'c', 'c'):
+        case VLC_FOURCC( 'M', '4', 'C', 'C'):
             break;
 
         default:
@@ -126,14 +145,10 @@ static int Open( vlc_object_t *p_this )
         msg_Err( p_dec, "out of memory" );
         return VLC_EGENERIC;
     }
-    p_sys->i_pts = 0;
-    p_sys->b_vop = VLC_FALSE;
-    p_sys->i_buffer = 0;
-    p_sys->i_buffer_size = 0;
-    p_sys->p_buffer = 0;
+    memset( p_sys, 0, sizeof(decoder_sys_t) );
 
     /* Setup properties */
-    p_dec->fmt_out = p_dec->fmt_in;
+    es_format_Copy( &p_dec->fmt_out, &p_dec->fmt_in );
     p_dec->fmt_out.i_codec = VLC_FOURCC( 'm', 'p', '4', 'v' );
 
     if( p_dec->fmt_in.i_extra )
@@ -145,7 +160,7 @@ static int Open( vlc_object_t *p_this )
                 p_dec->fmt_in.i_extra );
 
         msg_Dbg( p_dec, "opening with vol size:%d", p_dec->fmt_in.i_extra );
-        m4v_VOLParse( &p_dec->fmt_out,
+        m4v_VOLParse( p_dec, &p_dec->fmt_out,
                       p_dec->fmt_out.p_extra, p_dec->fmt_out.i_extra );
     }
     else
@@ -200,7 +215,7 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
 
     if( p_sys->i_buffer > 10*1000000 )
     {
-        msg_Err( p_dec, "mmh reseting context" );
+        msg_Warn( p_dec, "reseting context" );
         p_sys->i_buffer = 0;
     }
 
@@ -224,9 +239,10 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
         {
             /* Copy the complete VOL */
             p_dec->fmt_out.i_extra = p_start - p_vol;
-            p_dec->fmt_out.p_extra = malloc( p_dec->fmt_out.i_extra );
+            p_dec->fmt_out.p_extra =
+                realloc( p_dec->fmt_out.p_extra, p_dec->fmt_out.i_extra );
             memcpy( p_dec->fmt_out.p_extra, p_vol, p_dec->fmt_out.i_extra );
-            m4v_VOLParse( &p_dec->fmt_out,
+            m4v_VOLParse( p_dec, &p_dec->fmt_out,
                           p_dec->fmt_out.p_extra, p_dec->fmt_out.i_extra );
 
             p_vol = NULL;
@@ -247,13 +263,14 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
             p_sys->i_buffer -= i_out;
             p_start -= i_out;
 
-            /* FIXME do proper dts/pts */
-            p_out->i_pts = p_sys->i_pts;
-            p_out->i_dts = p_sys->i_dts;
+            p_out->i_flags = p_sys->i_flags;
+            p_out->i_pts = p_sys->i_interpolated_pts;
+            p_out->i_dts = p_sys->i_interpolated_dts;
+
             /* FIXME doesn't work when there is multiple VOP in one block */
-            if( p_block->i_dts > p_sys->i_dts )
+            if( p_block->i_dts > p_sys->i_interpolated_dts )
             {
-                p_out->i_length = p_block->i_dts - p_sys->i_dts;
+                p_out->i_length = p_block->i_dts - p_sys->i_interpolated_dts;
             }
 
             if( p_dec->fmt_out.i_extra > 0 )
@@ -266,11 +283,6 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
                 block_Release( p_out );
             }
 
-#if 0
-            fprintf( stderr, "pts=%lld dts=%lld length=%lldms\n",
-                     p_out->i_pts, p_out->i_dts,
-                     p_out->i_length / 1000 );
-#endif
             p_sys->b_vop = VLC_FALSE;
         }
 
@@ -279,22 +291,114 @@ static block_t *Packetize( decoder_t *p_dec, block_t **pp_block )
             /* Start of the VOL */
             p_vol = p_start;
         }
+        else if( p_start[3] == 0xb3 )
+        {
+            /* GOP header */
+        }
         else if( p_start[3] == 0xb6 )
         {
-            p_sys->b_vop = VLC_TRUE;
+            /* Parse the VOP */
+            bs_t s;
+            int i_modulo_time_base = 0;
+            int i_time_increment_bits;
+            int64_t i_time_increment, i_time_ref;
 
-            /* The pts information is not available in all the containers.
-             * FIXME: calculate the pts correctly */
-            if( p_block->i_pts > 0 )
+            /* FIXME: we don't actually check we received enough data to read
+             * the VOP time increment. */
+            bs_init( &s, &p_start[4],
+                     p_sys->i_buffer - (p_start - p_sys->p_buffer) - 4 );
+
+            switch( bs_read( &s, 2 ) )
             {
-                p_sys->i_pts = p_block->i_pts;
+                case 0:
+                    p_sys->i_flags = BLOCK_FLAG_TYPE_I;
+                    break;
+                case 1:
+                    p_sys->i_flags = BLOCK_FLAG_TYPE_P;
+                    break;
+                case 2:
+                    p_sys->i_flags = BLOCK_FLAG_TYPE_B;
+                    p_sys->b_frame = VLC_TRUE;
+                    break;
+                case 3: /* gni ? */
+                    p_sys->i_flags = BLOCK_FLAG_TYPE_PB;
+                    break;
+            }
+
+            while( bs_read( &s, 1 ) ) i_modulo_time_base++;
+            if( !bs_read1( &s ) ) continue; /* Marker */
+
+            /* VOP time increment */
+            i_time_increment_bits = vlc_log2(p_dec->p_sys->i_fps_num - 1) + 1;
+            if( i_time_increment_bits < 1 ) i_time_increment_bits = 1;
+            i_time_increment = bs_read( &s, i_time_increment_bits );
+
+            /* Interpolate PTS/DTS */
+            if( !(p_sys->i_flags & BLOCK_FLAG_TYPE_B) )
+            {
+                p_sys->i_last_time_ref = p_sys->i_time_ref;
+                p_sys->i_time_ref +=
+                    (i_modulo_time_base * p_dec->p_sys->i_fps_num);
+                i_time_ref = p_sys->i_time_ref;
             }
             else
             {
-                p_sys->i_pts = p_block->i_dts;
+                i_time_ref = p_sys->i_last_time_ref +
+                    (i_modulo_time_base * p_dec->p_sys->i_fps_num);
             }
-            p_sys->i_dts = p_block->i_dts;
+
+            if( p_dec->p_sys->i_fps_num < 5 && /* Work-around buggy streams */
+                p_dec->fmt_in.video.i_frame_rate > 0 &&
+                p_dec->fmt_in.video.i_frame_rate_base > 0 )
+            {
+                p_sys->i_interpolated_pts += I64C(1000000) *
+                    p_dec->fmt_in.video.i_frame_rate_base *
+                    p_block->i_rate / INPUT_RATE_DEFAULT /
+                    p_dec->fmt_in.video.i_frame_rate;
+            }
+            else if( p_dec->p_sys->i_fps_num )
+                p_sys->i_interpolated_pts +=
+                    ( I64C(1000000) * (i_time_ref + i_time_increment -
+                       p_sys->i_last_time - p_sys->i_last_timeincr) *
+                      p_block->i_rate / INPUT_RATE_DEFAULT /
+                      p_dec->p_sys->i_fps_num );
+
+            p_sys->i_last_time = i_time_ref;
+            p_sys->i_last_timeincr = i_time_increment;
+
+            /* Correct interpolated dts when we receive a new pts/dts */
+            if( p_block->i_pts > 0 )
+                p_sys->i_interpolated_pts = p_block->i_pts;
+            if( p_block->i_dts > 0 )
+                p_sys->i_interpolated_dts = p_block->i_dts;
+
+            if( (p_sys->i_flags & BLOCK_FLAG_TYPE_B) || !p_sys->b_frame )
+            {
+                /* Trivial case (DTS == PTS) */
+
+                p_sys->i_interpolated_dts = p_sys->i_interpolated_pts;
+
+                if( p_block->i_pts > 0 )
+                    p_sys->i_interpolated_dts = p_block->i_pts;
+                if( p_block->i_dts > 0 )
+                    p_sys->i_interpolated_dts = p_block->i_dts;
+
+                p_sys->i_interpolated_pts = p_sys->i_interpolated_dts;
+            }
+            else
+            {
+                if( p_sys->i_last_ref_pts > 0 )
+                    p_sys->i_interpolated_dts = p_sys->i_last_ref_pts;
+
+                p_sys->i_last_ref_pts = p_sys->i_interpolated_pts;
+            }
+
+            p_sys->b_vop = VLC_TRUE;
+
+            /* Don't re-use the same PTS/DTS twice */
+            p_block->i_pts = p_block->i_dts = 0;
         }
+
         p_start += 4; /* Next */
     }
 }
@@ -306,7 +410,8 @@ static int m4v_FindStartCode( uint8_t **pp_start, uint8_t *p_end )
 {
     uint8_t *p = *pp_start;
 
-    for( p = *pp_start; p < p_end - 4; p++ )
+    /* We wait for 4+1 bytes */
+    for( p = *pp_start; p < p_end - 5; p++ )
     {
         if( p[0] == 0 && p[1] == 0 && p[2] == 1 )
         {
@@ -353,14 +458,14 @@ static int vlc_log2( unsigned int v )
  *  TODO:
  *      - support aspect ratio
  */
-static int m4v_VOLParse( es_format_t *fmt, uint8_t *p_vol, int i_vol )
+static int m4v_VOLParse( decoder_t *p_dec, es_format_t *fmt,
+                         uint8_t *p_vol, int i_vol )
 {
     bs_t s;
     int i_vo_type;
     int i_vo_ver_id;
     int i_ar;
     int i_shape;
-    int i_time_increment_resolution;
 
     for( ;; )
     {
@@ -427,26 +532,21 @@ static int m4v_VOLParse( es_format_t *fmt, uint8_t *p_vol, int i_vol )
         bs_skip( &s, 4 );
     }
 
-    if( !bs_read1( &s ) )
-    {
-        /* marker */
-        return VLC_EGENERIC;
-    }
-    i_time_increment_resolution = bs_read( &s, 16 );
-    if( !bs_read1( &s ) )
-    {
-        /* marker */
-        return VLC_EGENERIC;
-    }
+    if( !bs_read1( &s ) ) return VLC_EGENERIC; /* Marker */
+
+    p_dec->p_sys->i_fps_num = bs_read( &s, 16 ); /* Time increment resolution*/
+    if( !p_dec->p_sys->i_fps_num ) p_dec->p_sys->i_fps_num = 1;
+
+    if( !bs_read1( &s ) ) return VLC_EGENERIC; /* Marker */
 
     if( bs_read1( &s ) )
     {
-        int i_time_increment_bits = vlc_log2( i_time_increment_resolution - 1 ) + 1;
-        if( i_time_increment_bits < 1 )
-        {
-            i_time_increment_bits = 1;
-        }
-        bs_skip( &s, i_time_increment_bits );
+        int i_time_increment_bits =
+            vlc_log2( p_dec->p_sys->i_fps_num - 1 ) + 1;
+
+        if( i_time_increment_bits < 1 ) i_time_increment_bits = 1;
+
+        p_dec->p_sys->i_fps_den = bs_read( &s, i_time_increment_bits );
     }
     if( i_shape == 0 )
     {
@@ -458,6 +558,3 @@ static int m4v_VOLParse( es_format_t *fmt, uint8_t *p_vol, int i_vol )
     }
     return VLC_SUCCESS;
 }
-
-
-
