@@ -2,7 +2,7 @@
  * events.c: Windows DirectX video output events handler
  *****************************************************************************
  * Copyright (C) 2001-2004 VideoLAN
- * $Id: events.c 9269 2004-11-10 13:04:45Z gbazin $
+ * $Id: events.c 10831 2005-04-26 14:27:47Z gbazin $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -81,7 +81,7 @@ static int DirectXConvertKey( int i_key );
  * The main goal of this thread is to isolate the Win32 PeekMessage function
  * because this one can block for a long time.
  *****************************************************************************/
-void DirectXEventThread( event_thread_t *p_event )
+void E_(DirectXEventThread)( event_thread_t *p_event )
 {
     MSG msg;
     POINT old_mouse_pos = {0,0}, mouse_pos;
@@ -105,12 +105,12 @@ void DirectXEventThread( event_thread_t *p_event )
     vlc_thread_ready( p_event );
 
     /* Set power management stuff */
-    if( (hkernel32 = GetModuleHandle( "KERNEL32" ) ) )
+    if( (hkernel32 = GetModuleHandle( _T("KERNEL32") ) ) )
     {
         ULONG (WINAPI* OurSetThreadExecutionState)( ULONG );
 
-        OurSetThreadExecutionState =
-            GetProcAddress( hkernel32, "SetThreadExecutionState" );
+        OurSetThreadExecutionState = (ULONG (WINAPI*)( ULONG ))
+            GetProcAddress( hkernel32, _T("SetThreadExecutionState") );
 
         if( OurSetThreadExecutionState )
             /* Prevent monitor from powering off */
@@ -298,16 +298,17 @@ void DirectXEventThread( event_thread_t *p_event )
             {
 #ifdef MODULE_NAME_IS_glwin32
                 SetWindowText( p_event->p_vout->p_sys->hwnd,
-                    VOUT_TITLE " (OpenGL output)" );
+                    _T(VOUT_TITLE) _T(" (OpenGL output)") );
 #else
                 if( p_event->p_vout->p_sys->b_using_overlay )
-                    SetWindowText( p_event->p_vout->p_sys->hwnd,
-                        VOUT_TITLE " (hardware YUV overlay DirectX output)" );
+                    SetWindowText( p_event->p_vout->p_sys->hwnd, _T(VOUT_TITLE)
+                        _T(" (hardware YUV overlay DirectX output)") );
                 else if( p_event->p_vout->p_sys->b_hw_yuv )
-                    SetWindowText( p_event->p_vout->p_sys->hwnd,
-                        VOUT_TITLE " (hardware YUV DirectX output)" );
-                else SetWindowText( p_event->p_vout->p_sys->hwnd,
-                        VOUT_TITLE " (software RGB DirectX output)" );
+                    SetWindowText( p_event->p_vout->p_sys->hwnd, _T(VOUT_TITLE)
+                        _T(" (hardware YUV DirectX output)") );
+                else
+                    SetWindowText( p_event->p_vout->p_sys->hwnd, _T(VOUT_TITLE)
+                        _T(" (software RGB DirectX output)") );
 #endif
             }
             else
@@ -357,10 +358,10 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
     HINSTANCE  hInstance;
     HMENU      hMenu;
     RECT       rect_window;
-    WNDCLASSEX wc;                            /* window class components */
+    WNDCLASS   wc;                            /* window class components */
     HICON      vlc_icon = NULL;
     char       vlc_path[MAX_PATH+1];
-    int        i_style;
+    int        i_style, i_stylex;
 
     msg_Dbg( p_vout, "DirectXCreateWindow" );
 
@@ -379,13 +380,14 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
 
     /* Get the Icon from the main app */
     vlc_icon = NULL;
+#ifndef UNDER_CE
     if( GetModuleFileName( NULL, vlc_path, MAX_PATH ) )
     {
         vlc_icon = ExtractIcon( hInstance, vlc_path, 0 );
     }
+#endif
 
     /* Fill in the window class structure */
-    wc.cbSize        = sizeof(WNDCLASSEX);
     wc.style         = CS_OWNDC|CS_DBLCLKS;          /* style: dbl click */
     wc.lpfnWndProc   = (WNDPROC)DirectXEventProc;       /* event handler */
     wc.cbClsExtra    = 0;                         /* no extra class data */
@@ -395,11 +397,10 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
     wc.hCursor       = LoadCursor(NULL, IDC_ARROW);    /* default cursor */
     wc.hbrBackground = GetStockObject(BLACK_BRUSH);  /* background color */
     wc.lpszMenuName  = NULL;                                  /* no menu */
-    wc.lpszClassName = "VLC DirectX";             /* use a special class */
-    wc.hIconSm       = vlc_icon;              /* load the vlc small icon */
+    wc.lpszClassName = _T("VLC DirectX");         /* use a special class */
 
     /* Register the window class */
-    if( !RegisterClassEx(&wc) )
+    if( !RegisterClass(&wc) )
     {
         WNDCLASS wndclass;
 
@@ -407,7 +408,7 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
 
         /* Check why it failed. If it's because one already exists
          * then fine, otherwise return with an error. */
-        if( !GetClassInfo( hInstance, "VLC DirectX", &wndclass ) )
+        if( !GetClassInfo( hInstance, _T("VLC DirectX"), &wndclass ) )
         {
             msg_Err( p_vout, "DirectXCreateWindow RegisterClass FAILED" );
             return VLC_EGENERIC;
@@ -415,14 +416,14 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
     }
 
     /* Register the video sub-window class */
-    wc.lpszClassName = "VLC DirectX video"; wc.hIconSm = 0; wc.hIcon = 0;
-    if( !RegisterClassEx(&wc) )
+    wc.lpszClassName = _T("VLC DirectX video"); wc.hIcon = 0;
+    if( !RegisterClass(&wc) )
     {
         WNDCLASS wndclass;
 
         /* Check why it failed. If it's because one already exists
          * then fine, otherwise return with an error. */
-        if( !GetClassInfo( hInstance, "VLC DirectX video", &wndclass ) )
+        if( !GetClassInfo( hInstance, _T("VLC DirectX video"), &wndclass ) )
         {
             msg_Err( p_vout, "DirectXCreateWindow RegisterClass FAILED" );
             return VLC_EGENERIC;
@@ -437,20 +438,36 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
     rect_window.left   = 10;
     rect_window.right  = rect_window.left + p_vout->p_sys->i_window_width;
     rect_window.bottom = rect_window.top + p_vout->p_sys->i_window_height;
-    AdjustWindowRect( &rect_window, WS_OVERLAPPEDWINDOW|WS_SIZEBOX, 0 );
 
-    i_style = WS_OVERLAPPEDWINDOW|WS_SIZEBOX|WS_VISIBLE|WS_CLIPCHILDREN;
+    if( var_GetBool( p_vout, "video-deco" ) )
+    {
+        /* Open with window decoration */
+        AdjustWindowRect( &rect_window, WS_OVERLAPPEDWINDOW|WS_SIZEBOX, 0 );
+        i_style = WS_OVERLAPPEDWINDOW|WS_SIZEBOX|WS_VISIBLE|WS_CLIPCHILDREN;
+        i_stylex = 0;
+    }
+    else
+    {
+        /* No window decoration */
+        AdjustWindowRect( &rect_window, WS_POPUP, 0 );
+        i_style = WS_POPUP|WS_VISIBLE|WS_CLIPCHILDREN;
+        i_stylex = 0; // WS_EX_TOOLWINDOW; Is TOOLWINDOW really needed ?
+                      // It messes up the fullscreen window.
+    }
 
     if( p_vout->p_sys->hparent )
     {
         i_style = WS_VISIBLE|WS_CLIPCHILDREN|WS_CHILD;
+        i_stylex = 0;
     }
 
+    p_vout->p_sys->i_window_style = i_style;
+
     /* Create the window */
     p_vout->p_sys->hwnd =
-        CreateWindowEx( WS_EX_NOPARENTNOTIFY,
-                    "VLC DirectX",                   /* name of window class */
-                    VOUT_TITLE " (DirectX Output)", /* window title bar text */
+        CreateWindowEx( WS_EX_NOPARENTNOTIFY | i_stylex,
+                    _T("VLC DirectX"),               /* name of window class */
+                    _T(VOUT_TITLE) _T(" (DirectX Output)"),  /* window title */
                     i_style,                                 /* window style */
                     (p_vout->p_sys->i_window_x < 0) ? CW_USEDEFAULT :
                         p_vout->p_sys->i_window_x,   /* default X coordinate */
@@ -483,8 +500,8 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
 
         /* Create our fullscreen window */
         p_vout->p_sys->hfswnd =
-            CreateWindowEx( WS_EX_APPWINDOW, "VLC DirectX",
-                            VOUT_TITLE " (DirectX Output)",
+            CreateWindowEx( WS_EX_APPWINDOW, _T("VLC DirectX"),
+                            _T(VOUT_TITLE) _T(" (DirectX Output)"),
                             WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN|WS_SIZEBOX,
                             CW_USEDEFAULT, CW_USEDEFAULT,
                             CW_USEDEFAULT, CW_USEDEFAULT,
@@ -501,9 +518,9 @@ static int DirectXCreateWindow( vout_thread_t *p_vout )
 
     /* Append a "Always On Top" entry in the system menu */
     hMenu = GetSystemMenu( p_vout->p_sys->hwnd, FALSE );
-    AppendMenu( hMenu, MF_SEPARATOR, 0, "" );
+    AppendMenu( hMenu, MF_SEPARATOR, 0, _T("") );
     AppendMenu( hMenu, MF_STRING | MF_UNCHECKED,
-                       IDM_TOGGLE_ON_TOP, "Always on &Top" );
+                       IDM_TOGGLE_ON_TOP, _T("Always on &Top") );
 
     return VLC_SUCCESS;
 }
@@ -537,7 +554,7 @@ static void DirectXCloseWindow( vout_thread_t *p_vout )
  * its job is to update the source and destination RECTs used to display the
  * picture.
  *****************************************************************************/
-void DirectXUpdateRects( vout_thread_t *p_vout, vlc_bool_t b_force )
+void E_(DirectXUpdateRects)( vout_thread_t *p_vout, vlc_bool_t b_force )
 {
 #define rect_src p_vout->p_sys->rect_src
 #define rect_src_clipped p_vout->p_sys->rect_src_clipped
@@ -673,7 +690,7 @@ void DirectXUpdateRects( vout_thread_t *p_vout, vlc_bool_t b_force )
     rect_dest_clipped.bottom -= p_vout->p_sys->rect_display.top;
 
     if( p_vout->p_sys->b_using_overlay )
-        DirectXUpdateOverlay( p_vout );
+        E_(DirectXUpdateOverlay)( p_vout );
 
     /* Signal the change in size/position */
     p_vout->p_sys->i_changes |= DX_POSITION_CHANGE;
@@ -711,6 +728,14 @@ static long FAR PASCAL DirectXEventProc( HWND hwnd, UINT message,
         p_vout = (vout_thread_t *)GetWindowLongPtr( hwnd, GWLP_USERDATA );
     }
 
+    /* Catch the screensaver and the monitor turn-off */
+    if( message == WM_SYSCOMMAND &&
+        ( wParam == SC_SCREENSAVE || wParam == SC_MONITORPOWER ) )
+    {
+        //if( p_vout ) msg_Dbg( p_vout, "WinProc WM_SYSCOMMAND screensaver" );
+        return 0; /* this stops them from happening */
+    }
+
     if( !p_vout )
     {
         /* Hmmm mozilla does manage somehow to save the pointer to our
@@ -725,7 +750,7 @@ static long FAR PASCAL DirectXEventProc( HWND hwnd, UINT message,
     {
 
     case WM_WINDOWPOSCHANGED:
-        DirectXUpdateRects( p_vout, VLC_TRUE );
+        E_(DirectXUpdateRects)( p_vout, VLC_TRUE );
         return 0;
 
     /* the user wants to close the window */
@@ -754,23 +779,16 @@ static long FAR PASCAL DirectXEventProc( HWND hwnd, UINT message,
     case WM_SYSCOMMAND:
         switch (wParam)
         {
-            case SC_SCREENSAVE:                     /* catch the screensaver */
-            case SC_MONITORPOWER:              /* catch the monitor turn-off */
-                msg_Dbg( p_vout, "WinProc WM_SYSCOMMAND" );
-                return 0;                  /* this stops them from happening */
             case IDM_TOGGLE_ON_TOP:            /* toggle the "on top" status */
             {
                 vlc_value_t val;
                 msg_Dbg( p_vout, "WinProc WM_SYSCOMMAND: IDM_TOGGLE_ON_TOP");
 
-                /* Get the current value... */
-                if( var_Get( p_vout, "video-on-top", &val ) < 0 )
-                    return 0;
-                /* ...and change it */
+                /* Change the current value */
+                var_Get( p_vout, "video-on-top", &val );
                 val.b_bool = !val.b_bool;
                 var_Set( p_vout, "video-on-top", val );
                 return 0;
-                break;
             }
         }
         break;
@@ -778,7 +796,7 @@ static long FAR PASCAL DirectXEventProc( HWND hwnd, UINT message,
     case WM_VLC_CREATE_VIDEO_WIN:
         /* Create video sub-window */
         p_vout->p_sys->hvideownd =
-            CreateWindow( "VLC DirectX video", "",      /* window class text */
+            CreateWindow( _T("VLC DirectX video"), _T(""),   /* window class */
                     WS_CHILD | WS_VISIBLE,                   /* window style */
                     CW_USEDEFAULT, CW_USEDEFAULT,     /* default coordinates */
                     CW_USEDEFAULT, CW_USEDEFAULT,
@@ -835,6 +853,9 @@ static struct
     { VK_PRIOR, KEY_PAGEUP },
     { VK_NEXT, KEY_PAGEDOWN },
 
+    { VK_INSERT, KEY_INSERT },
+    { VK_DELETE, KEY_DELETE },
+
     { VK_CONTROL, 0 },
     { VK_SHIFT, 0 },
     { VK_MENU, 0 },
@@ -879,7 +900,7 @@ static int Control( vout_thread_t *p_vout, int i_query, va_list args )
         rect_window.top = rect_window.left = 0;
         rect_window.right  = p_vout->i_window_width * f_arg;
         rect_window.bottom = p_vout->i_window_height * f_arg;
-        AdjustWindowRect( &rect_window, WS_OVERLAPPEDWINDOW|WS_SIZEBOX, 0 );
+        AdjustWindowRect( &rect_window, p_vout->p_sys->i_window_style, 0 );
 
         SetWindowPos( p_vout->p_sys->hwnd, 0, 0, 0,
                       rect_window.right - rect_window.left,
@@ -899,10 +920,13 @@ static int Control( vout_thread_t *p_vout, int i_query, va_list args )
         point.x = point.y = 0;
         ClientToScreen( p_vout->p_sys->hwnd, &point );
 
-        SetParent( p_vout->p_sys->hwnd, GetDesktopWindow() );
+        SetParent( p_vout->p_sys->hwnd, 0 );
+        p_vout->p_sys->i_window_style =
+            WS_CLIPCHILDREN | WS_OVERLAPPEDWINDOW | WS_SIZEBOX;
         SetWindowLong( p_vout->p_sys->hwnd, GWL_STYLE,
-                       WS_CLIPCHILDREN | WS_OVERLAPPEDWINDOW |
-                       WS_SIZEBOX | (i_query == VOUT_CLOSE ? 0 : WS_VISIBLE) );
+                       p_vout->p_sys->i_window_style |
+                       (i_query == VOUT_CLOSE ? 0 : WS_VISIBLE) );
+        SetWindowLong( p_vout->p_sys->hwnd, GWL_EXSTYLE, WS_EX_APPWINDOW );
         SetWindowPos( p_vout->p_sys->hwnd, 0, point.x, point.y, 0, 0,
                       SWP_NOSIZE|SWP_NOZORDER|SWP_FRAMECHANGED );
 
